package org.embulk.input.filename;

import com.google.common.collect.ImmutableList;

import org.embulk.config.ConfigSource;
import org.embulk.config.ConfigDiff;
import org.embulk.test.EmbulkTests;
import org.embulk.test.TestingEmbulk;
import org.embulk.spi.InputPlugin;
import org.embulk.spi.ParserPlugin;
import org.embulk.spi.SchemaConfig;
import org.embulk.spi.ColumnConfig;
import org.junit.Rule;
import org.junit.Before;
import org.junit.Test;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.Files;
import java.io.File;
import java.io.IOException;
import java.util.List;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;

import static org.embulk.test.EmbulkTests.readSortedFile;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertThat;

public class TestFilenameFileInputPlugin
{
  private static ConfigSource loadYamlResource(TestingEmbulk embulk, String filename) throws Exception{
    // This function help load the config yml file.
    return embulk.loadYamlResource(filename);
  }

  

  @Rule
  public TestingEmbulk embulk = TestingEmbulk.builder()
          .registerPlugin(InputPlugin.class,"filename",FilenameFileInputPlugin.class)
          .registerPlugin(ParserPlugin.class,"none-bin",NoneBinParserPlugin.class)
          .build();

  @Test
  public void test() throws Exception{
    File rootFile = new File(TestFilenameFileInputPlugin.class.getResource("/test.yml").toURI()).getParentFile();
    String rootPath = rootFile.getAbsolutePath();
    System.out.println("This is the root of the resources: "+rootPath);
    Path out1 = Paths.get(rootPath+"/output.csv");
    //We can load the yml file in the resource or just define the config below
    //ConfigSource config = loadYamlResource(embulk,"/test.yml");
    //config = config.set("path_prefix",rootPath+"/data/test.csv");
    ConfigSource config  = embulk.newConfig()
         .set("type","filename")
         .set("path_prefix",rootPath+"/data/test.csv")
         .set("parser",embulk.newConfig()
                 .set("charset","UTF-8")
                 .set("newline","CRLF")
                 .set("type","csv")
                 .set("delimiter",",")
                 .set("quote","")
                 .set("columns",newSchemaConfig("filename:string")));
    //System.out.println(config);
    TestingEmbulk.RunResult result1 = embulk.runInput(config,out1);

    try {
      List<String> sourceLines = Files.readAllLines(Paths.get(rootPath+"/data/test.csv"));
      List<String> targetLines = Files.readAllLines(Paths.get(rootPath+"/output.csv"));
      char zero = (char) 0;
      assertEquals(targetLines.get(0),rootPath+"/data/test.csv"+zero);
      //assertEquals(targetLines.get(0).trim(),rootPath+"/data/test.csv");
      assertEquals(targetLines.size(),sourceLines.size());
      for(int i = 1; i<sourceLines.size(); i++){
        assertEquals(targetLines.get(i),sourceLines.get(i));
      }
    } catch (IOException ex){ex.printStackTrace();}

  }

  @Test
  public void testModifiedOrder() throws Exception{
//    ConfigSource config = embulk.loadYamlResource("testModifiedOrder.yml");
    //System.out.println("TestOrder: " + config.toString());
    Path out1 = embulk.createTempFile("csv");

    ConfigSource execConfig  = embulk.newConfig()
          .set("max_threads","1");


    Path path_src =  Paths.get("build/resources/test/testModifiedOrder");
    
    Files.list(path_src).forEach(a -> System.out.println(a.toString()));
    Files.list(out1.getParent()).forEach(a -> System.out.println(a.toString()));
  
    ConfigSource config  = embulk.newConfig()
          .set("type","filename")
          .set("path_prefix",path_src.toAbsolutePath().toString()+"/sample_")
          .set("parser",embulk.newConfig()
             .set("charset","UTF-8")
             .set("newline","CRLF")
             .set("type","csv")
             .set("delimiter",",")
             .set("quote","")
             .set("columns",newSchemaConfig("filename:string")));
    System.out.println("The tmp: "+ out1+ " # " +  Files.isDirectory(out1)+ " # " + Files.isRegularFile(out1));
    System.out.println("the config : " + config);
    TestingEmbulk.RunResult result2 =  embulk.runInput(config,out1,execConfig);
  
    List<Path> arrayOut = new ArrayList<Path> ();
    Files.list(out1.getParent()).forEach(a ->{
                     try { if(Files.isDirectory(a)) { Files.list(a).forEach(b -> arrayOut.add(b)); }}
                     catch (IOException ex) {ex.printStackTrace();} 
                   });

    System.out.println("Not sorted yet " + arrayOut);
    Collections.sort(arrayOut,new Comparator<Path>(){
      @Override
       public int compare(Path p1,Path p2){
        int res = (int) Long.valueOf(p1.toFile().lastModified()).compareTo(p2.toFile().lastModified());
        System.out.println("The result is :" + res+" # " +  p1.toString() +" # " + p2.toString());
        return res;
       }
    });
    for (Path p : arrayOut){
      System.out.println(p.toFile().lastModified());


    }
    System.out.println(arrayOut);
    
     for (Path  p : arrayOut){
       System.out.println(p.toString());
       try{
         List<String> lines = Files.readAllLines(p);
         for (String line : lines){System.out.println(line);}
       } catch (IOException ex) {ex.printStackTrace();}
     }
    //Files.list(out1).forEach(a -> System.out.println("The Source files: " + a.toString()));
  }
  public SchemaConfig newSchemaConfig(String...configs){
    ImmutableList.Builder<ColumnConfig> schema = ImmutableList.builder();
    for (String column: configs){
      ColumnConfig columnConfig = newColumnConfig(column);
      if (columnConfig != null){
         schema.add(columnConfig);
      }
    }
    return new SchemaConfig(schema.build());
  }

  public ColumnConfig newColumnConfig(String column){
    String[] tuple = column.split(":",2);
    return new ColumnConfig(embulk.newConfig()
            .set("name",tuple[0])
            .set("type",tuple[1]));
  }
}
